/*
This is a part of the One LiteStep module source code.

Copyright (C) 2002 Erik Christiansson, aka Sci
erik@alphafish.com
www.alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "one.h"
#include "strsafe.h"

SAFE_CHAR_H(g_szClosed, MAX_BANGARGS);
SAFE_CHAR_H(g_szOpen, MAX_BANGARGS);
SAFE_CHAR_H(g_szActive, MAX_BANGARGS);

int initModuleEx(HWND /* hParent */, HINSTANCE /* hInstance */, LPCSTR /* szPath */)
{
	GetRCLine("OneClosedCmd", g_szClosed, MAX_BANGARGS, ".none");
	GetRCLine("OneOpenCmd", g_szOpen, MAX_BANGARGS, ".focus");
	GetRCLine("OneActiveCmd", g_szActive, MAX_BANGARGS, "");
	
	AddBangCommand("!One", bangOne);

	return 0;
}

void quitModule(HINSTANCE /* hInstance */)
{
	RemoveBangCommand("!One");
}

void ProcessArgument(HWND hWnd, HWND hCaller, LPCSTR pszArgument)
{
	if (!stricmp(pszArgument, ".none"))
	{
		return;
	}
	else if (!stricmp(pszArgument, ".focus"))
	{
		SwitchToThisWindow(hWnd, TRUE);
	}
	else if (!stricmp(pszArgument, ".minimize"))
	{
		SendMessage(hWnd, WM_SYSCOMMAND, SC_MINIMIZE, NULL);
	}
	else if (!stricmp(pszArgument, ".maximize"))
	{
		SendMessage(hWnd, WM_SYSCOMMAND, SC_MAXIMIZE, NULL);
	}
	else if (!stricmp(pszArgument, ".restore"))
	{
		SendMessage(hWnd, WM_SYSCOMMAND, SC_RESTORE, NULL);
	}
	else if (!stricmp(pszArgument, ".close"))
	{
		SendMessage(hWnd, WM_SYSCOMMAND, SC_CLOSE, NULL);
	}
	else if (!stricmp(pszArgument, ".show"))
	{
		ShowWindow(hWnd,SW_SHOW);
	}
	else if (!stricmp(pszArgument, ".hide"))
	{
		ShowWindow(hWnd,SW_HIDE);
	}
	else
	{
		LSExecute(hCaller, pszArgument, NULL);
	}
}

void bangOne(HWND hCaller, LPCSTR pszArgs)
{
	if (IsValidStringPtr(pszArgs))
	{
//		BOOL (WINAPI *SwitchToThisWindow)(HWND, BOOL) = (BOOL (WINAPI *)(HWND, BOOL)) GetProcAddress(GetModuleHandle("USER32.DLL"), "SwitchToThisWindow");
		
		HWND hWnd = NULL;

		SAFE_CHAR(szClass, MAX_BANGARGS);
		SAFE_CHAR(szTitle, MAX_BANGARGS);
		SAFE_CHAR(szOpen, MAX_BANGARGS);
		SAFE_CHAR(szClosed, MAX_BANGARGS);
		SAFE_CHAR(szActive, MAX_BANGARGS);
		
		LPSTR lpszTokens[] = { szClass, szTitle, szClosed, szOpen, szActive };
		
		CommandTokenize(pszArgs, lpszTokens, 5, NULL);

		if (!strlen(szClosed))
		{
			StringCchCopy(szClosed, MAX_BANGARGS, g_szClosed);
		}

		if (!strlen(szOpen))
		{
			StringCchCopy(szOpen, MAX_BANGARGS, g_szOpen);
		}

		if (!strlen(szActive))
		{
			StringCchCopy(szActive, MAX_BANGARGS, strlen(g_szActive) ? g_szActive : szOpen);
		}
		
		hWnd = FindWindow( strlen(szClass) ? szClass : NULL,
						   strlen(szTitle) ? szTitle : NULL);

		if (!hWnd)
		{
			LSExecute(hCaller, szClosed, SW_SHOW);
		}
		else
		{
			if (hWnd == GetForegroundWindow())
			{
				ProcessArgument(hWnd, hCaller, szActive);
			}
			else
			{
				ProcessArgument(hWnd, hCaller, szOpen);
			}
		}
		
	}
}
